"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const abstract_command_1 = require("../abstract-command");
const file_operations_1 = require("../../utils/file-operations");
const service_files_constants_1 = require("./service-files-constants");
const path = require("path");
const logger_1 = require("../../utils/logger");
const parser = require("typescript-parser");
const _ = require("lodash");
const fileDefinitionMap = new Map([
    [
        'injector',
        {
            filePattern: service_files_constants_1.injectorPattern,
            declarationName: 'injector',
            possibleStartSeparators: service_files_constants_1.injectorStartSeparators,
            definitionSignatureFn: service_files_constants_1.injectorSignatureFn,
            ignoreContext: false
        }
    ],
    [
        'context',
        {
            filePattern: service_files_constants_1.contextPattern,
            declarationName: 'getContext',
            possibleStartSeparators: service_files_constants_1.contextStartSeparators,
            definitionSignatureFn: service_files_constants_1.contextSignatureFn,
            ignoreContext: true
        }
    ],
    [
        'context-interface',
        {
            filePattern: service_files_constants_1.contextInterfacePattern,
            declarationName: 'IAppContext',
            possibleStartSeparators: service_files_constants_1.contextInterfaceStartSeparators,
            definitionSignatureFn: service_files_constants_1.contextInterfaceSignatureFn,
            ignoreContext: true
        }
    ]
]);
class Service extends abstract_command_1.AbstractCommand {
    constructor() {
        super();
        this.parser = new parser.TypescriptParser();
    }
    getName() {
        return 'service [service-path]';
    }
    getDescription() {
        return 'Add new service to services dir, and to server gql context';
    }
    getAlias() {
        return 's';
    }
    getOptions() {
        return {
            optionsPattern: ['--ignoreContext'],
            optionsDescription: ['Do not add service to server gql context']
        };
    }
    getAction() {
        return (servicePath, cmd) => __awaiter(this, void 0, void 0, function* () {
            return this.createServiceFile(servicePath, cmd.parent.ignoreContext);
        });
    }
    createServiceFile(servicePath, ignoreContext) {
        return __awaiter(this, void 0, void 0, function* () {
            yield this.writeServiceFile(servicePath, ignoreContext);
            logger_1.default.info('Service generation command was completed');
        });
    }
    writeServiceFile(servicePath, ignoreContext) {
        return __awaiter(this, void 0, void 0, function* () {
            try {
                const pathToServicesDir = yield file_operations_1.locateFile(service_files_constants_1.servicePattern, './src', 'dir');
                const commonRootPath = file_operations_1.findCommonPath(pathToServicesDir);
                if (!service_files_constants_1.servicePattern.test(commonRootPath)) {
                    logger_1.default.warn('services directory should appear exactly once in the project, found paths- ' +
                        pathToServicesDir.toString());
                    logger_1.default.warn('please make sure you are at the project root directory and that src dir contains single services dir');
                }
                else {
                    const joinedPathToService = path.join(pathToServicesDir[0], servicePath);
                    const serviceFileExists = yield file_operations_1.fileExists(joinedPathToService);
                    if (serviceFileExists) {
                        logger_1.default.warn('Service file already exists. Please select a different location');
                    }
                    else {
                        logger_1.default.info('Generating service file...');
                        const serviceName = this.getServiceClassName(servicePath);
                        yield file_operations_1.writeToFile(joinedPathToService, this.getServiceFileContent(serviceName));
                        logger_1.default.info('Service file was added to- ' + joinedPathToService);
                        logger_1.default.info('Editing service definition files...');
                        const definitionFileTypes = Array.from(fileDefinitionMap.keys());
                        const serviceDefinitionPromises = definitionFileTypes
                            .filter((def) => {
                            const definitionProps = fileDefinitionMap.get(def);
                            return ignoreContext ? !definitionProps.ignoreContext : true;
                        })
                            .map(def => this.modifyServiceDefinitionFile(joinedPathToService, serviceName, def));
                        yield Promise.all(serviceDefinitionPromises);
                        logger_1.default.info('Definition files were successfully updated!');
                    }
                }
            }
            catch (err) {
                logger_1.default.error(`Got error in generating service- ${err}`);
            }
        });
    }
    getServiceFileContent(serviceClassName) {
        return service_files_constants_1.serviceFilePrefix + serviceClassName + service_files_constants_1.serviceFileSuffix;
    }
    getServiceClassName(servicePath) {
        return (_.startCase(path.parse(servicePath).base.split('.')[0]).replace(/\W/g, '') +
            service_files_constants_1.serviceFileNameSuffix);
    }
    modifyServiceDefinitionFile(pathToService, serviceClassName, definitionFileType) {
        return __awaiter(this, void 0, void 0, function* () {
            logger_1.default.info('Modifying definitions in ' + definitionFileType + ' file...');
            const filePatternToLocate = fileDefinitionMap.get(definitionFileType)
                .filePattern;
            const pathToDefFile = yield file_operations_1.locateFile(filePatternToLocate, './src', 'file');
            if (!pathToDefFile || pathToDefFile.length !== 1) {
                logger_1.default.warn('Definition file for ' + definitionFileType + ' not found');
            }
            else {
                const fileContent = yield this.getModifiedFileContent(pathToService, serviceClassName, pathToDefFile[0], definitionFileType);
                return file_operations_1.writeToFile(pathToDefFile[0], fileContent);
            }
        });
    }
    getModifiedFileContent(pathToService, serviceClassName, defFilePath, definitionFileType) {
        return __awaiter(this, void 0, void 0, function* () {
            const defFileContent = yield file_operations_1.readFileContent(defFilePath);
            const parsedFile = yield this.parser.parseFile(defFilePath, './');
            const { content: importsContent, end: importEnd } = this.getImportDefinitions(pathToService, serviceClassName, parsedFile, defFileContent);
            const { content: defContent, start: defStart, end: defEnd } = this.getModifiedServiceDefinitions(serviceClassName, parsedFile, defFileContent, definitionFileType);
            return (importsContent +
                defFileContent.substring(importEnd, defStart) +
                defContent +
                defFileContent.substring(defEnd));
        });
    }
    getImportDefinitions(pathToService, serviceClassName, parsedFile, defFileContent) {
        const importsEndChar = Array.isArray(parsedFile.imports)
            ? parsedFile.imports[parsedFile.imports.length - 1].end || 0
            : 0;
        const originalImports = defFileContent.substring(0, importsEndChar);
        return {
            content: originalImports + '\n' + this.getServiceImportLine(pathToService, serviceClassName),
            start: 0,
            end: importsEndChar
        };
    }
    getServiceImportLine(pathToService, serviceClassName) {
        return ('import { ' +
            serviceClassName +
            " } from '" +
            service_files_constants_1.importServicePrefix +
            this.locationInsideServicesDir(pathToService) +
            "'");
    }
    locationInsideServicesDir(pathToService) {
        return pathToService
            .split('.')[0]
            .split('services')[1]
            .replace(/\\/g, '/');
    }
    getModifiedServiceDefinitions(serviceClassName, parsedFile, defFileContent, definitionFileType) {
        const declarationsArray = parsedFile.declarations;
        const { content: definitionToModify, start, end } = this.getDeclarationContentByDefType(declarationsArray, defFileContent, definitionFileType);
        return {
            content: this.addServiceToPreviousDefinition(serviceClassName, definitionToModify, definitionFileType),
            start,
            end
        };
    }
    getDeclarationContentByDefType(declarationsArray, defFileContent, definitionFileType) {
        const emptyDeclarationInfo = {
            content: '',
            start: 0,
            end: 0
        };
        const nameToFilterBy = fileDefinitionMap.get(definitionFileType)
            .declarationName;
        const filteredDeclaration = declarationsArray.filter(declaration => declaration.name === nameToFilterBy && declaration.start);
        if (filteredDeclaration.length !== 1) {
            logger_1.default.warn('Definition file ' +
                definitionFileType +
                ' should contain exactly one definition for ' +
                nameToFilterBy +
                ', file will not be changed...');
            return emptyDeclarationInfo;
        }
        else {
            const declarationToFind = filteredDeclaration[0];
            if (!declarationToFind.start || !declarationToFind.end) {
                logger_1.default.warn('Could not find start and end for declaration ' + declarationToFind.name);
                return emptyDeclarationInfo;
            }
            else {
                return {
                    content: defFileContent.substring(declarationToFind.start, declarationToFind.end),
                    start: declarationToFind.start,
                    end: declarationToFind.end
                };
            }
        }
    }
    addServiceToPreviousDefinition(serviceClassName, definitionToModify, definitionFileType) {
        const definitionFileProps = fileDefinitionMap.get(definitionFileType);
        const possibleStartSeparators = definitionFileProps.possibleStartSeparators;
        const serviceSignatureFn = definitionFileProps.definitionSignatureFn;
        for (let separator of possibleStartSeparators) {
            const indexOfSeparator = definitionToModify.lastIndexOf(separator);
            if (indexOfSeparator !== -1) {
                return (definitionToModify.substring(0, indexOfSeparator + separator.length) +
                    '\n' +
                    serviceSignatureFn(serviceClassName) +
                    definitionToModify.substring(indexOfSeparator + separator.length));
            }
        }
        return definitionToModify;
    }
}
exports.Service = Service;
//# sourceMappingURL=service.js.map